#include "fs.hh"

#include <fcntl.h>

class temp_file {
public:
  temp_file( char const * const name ) :
	name( name )
  {
	this->fd = open( name, O_CREAT | O_EXCL | O_RDWR, 0700 );
  }

  ~temp_file()
  {
	this->close();
	::unlink( this->name );
  }

  void close()
  {
	if( this->fd ) {
	  ::close( this->fd );
	  this->fd = 0;
	}
  }

  int fd;
  char const * const name;
};

int test_normalisation( char const * const real_name, char const * const alias ) {
  temp_file f( real_name );
  if( !f.fd ) {
	puts( "file already exists!" );
	return -1;
  }

  char const * const message = "I am a message";
  int length = strlen( message ) + 1;
  write( f.fd, message, length );
  f.close();

  printf( "successfully created file (%s)\n", real_name );

  std::string normalised;
  platform_fs_normalisation_hook( alias, normalised );

  int fd = open( normalised.c_str(), O_RDONLY );
  if( !fd ) {
	puts( "could not open the normalised file" );
	return -1;
  }

  printf( "successfully opened alias (%s) as normalised (%s)\n", alias, normalised.c_str() );

  char * const buffer = (char*) malloc( length );
  read( fd, buffer, length );
  if( strcmp( buffer, message )  ) {
	puts( "messages are different!" );
	puts( message );
	puts( buffer );
	return -1;
  }
  else {
	puts( "successful normalisation!" );
	return 0;
  }  
}

int main( int argc, char * argv[] ) {
  test_normalisation( "abc", "ABC" );
  test_normalisation( "/Users/peter/Programming/test_paths/abc", "/Users/peter/Programming/test_paths/ABC" );
  return 0;
}
